/**
 *
 */
package de.intevation.gnv.state.profile.horizontalcrosssection;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;

import org.apache.log4j.Logger;

import org.jfree.chart.ChartTheme;

import au.com.bytecode.opencsv.CSVWriter;
import de.intevation.gnv.chart.Chart;
import de.intevation.gnv.chart.ChartLabels;
import de.intevation.gnv.geobackend.base.Result;
import de.intevation.gnv.state.InputData;
import de.intevation.gnv.state.exception.StateException;
import de.intevation.gnv.state.timeseries.TimeSeriesOutputState;
import de.intevation.gnv.statistics.Statistics;

import de.intevation.artifacts.CallContext;

/**
 * @author Tim Englich <tim.englich@intevation.de>
 * 
 */
public class HorizontalCrossSectionMeshOutputState 
                                      extends TimeSeriesOutputState {

    private static Logger log = Logger
    .getLogger(HorizontalCrossSectionMeshOutputState.class);
    
    /**
     * The UID of this Class
     */
    private static final long serialVersionUID = 3233620652465061860L;

    /**
     * Constructor
     */
    public HorizontalCrossSectionMeshOutputState() {
        super();
        super.domainLable = "chart.horizontalcrosssection.title.xaxis";
    }

    @Override
    protected Chart getChart(
        ChartLabels  chartLables,
        ChartTheme   theme,
        Collection   parameters,
        Collection   measurements,
        Collection   dates,
        Object       result,
        Locale       locale,
        String       uuid,
        boolean      linesVisible,
        boolean      shapesVisible,
        CallContext  callContext
    ) {
        Chart chart = null;

        if (CACHE_CHART) {
            log.info("Try to get horizontalcrosssection chart from cache.");
            chart = (Chart) getChartFromCache(uuid, callContext);
        }

        if (chart != null)
            return chart;

        log.info("Chart not in cache yet.");
        
        log.warn("This sort of chart is not implemented yet.");
        /* TODO Implement a special chart for this sort of charts.
        chart = new HorizontalProfileChart(
            chartLables,
            chartTheme,
            parameters,
            measurements,
            result,
            dates,
            locale
        );
        chart.generateChart();

        if (CACHE_CHART) {
            log.info("Put chart into cache.");
            purifyChart(chart, uuid);
        }
        */
        
        InputData meshPolygon = inputData.get("mesh_polygon");
        String meshPolygonWkt = null;
        if (meshPolygon != null){
            meshPolygonWkt = meshPolygon.getValue();
            
        }
        log.debug("Used Polygon: "+meshPolygonWkt);

        return chart;
    }


    /**
     * @see de.intevation.gnv.state.timeseries.TimeSeriesOutputState#getStatisticsGenerator()
     */
    @Override
    protected Statistics getStatisticsGenerator() {
        return null; //Statistics are not supported for this kind of OutputState.
    }
    
    /**
     * @see de.intevation.gnv.state.timeseries.TimeSeriesOutputState#createCSV(java.io.OutputStream,
     *      java.util.Collection)
     */
    @Override
    protected void createCSV(OutputStream outputStream,
                             Collection<Result> chartResult)
                                            throws UnsupportedEncodingException,
                                                   IOException,
                                                   StateException {
        if (chartResult != null) {
            try {
                CSVWriter writer = new CSVWriter(new OutputStreamWriter(
                        outputStream, "ISO-8859-1"), ','); 
                // USE THIS ENCODING BECAUSE OF
                // PROBLEMS WITH EXCEL AND UTF-8
                Iterator<Result> it = chartResult.iterator();
                while (it.hasNext()) {
                    Result result = it.next();
                    int i = 0;
                    String[] entries = new String[5];
                    entries[i++] = result.getString("SHAPE");
                    entries[i++] = result.getString("YORDINATE");
                    entries[i++] = result.getString("IPOSITION");
                    entries[i++] = result.getString("JPOSITION");
                    entries[i++] = result.getString("KPOSITION");
                    writer.writeNext(entries);
                }
                writer.close();
            } catch (Exception e) {
                log.error(e,e);
                throw new StateException(
                "Exception occured while parsing an Point from WKT.");
            }
        } else {
            log.error("No Data given for generating an CSV-File.");
            throw new StateException(
                    "No Data given for generating an CSV-File.");
        }
    }

}
